# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/transaction_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import ledger_offset_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2
from . import transaction_filter_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2
from . import transaction_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0com/daml/ledger/api/v1/transaction_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/ledger_offset.proto\x1a/com/daml/ledger/api/v1/transaction_filter.proto\x1a(com/daml/ledger/api/v1/transaction.proto\"\x86\x02\n\x16GetTransactionsRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12:\n\x05\x62\x65gin\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x05\x62\x65gin\x12\x36\n\x03\x65nd\x18\x03 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x03\x65nd\x12\x41\n\x06\x66ilter\x18\x04 \x01(\x0b\x32).com.daml.ledger.api.v1.TransactionFilterR\x06\x66ilter\x12\x18\n\x07verbose\x18\x05 \x01(\x08R\x07verbose\"b\n\x17GetTransactionsResponse\x12G\n\x0ctransactions\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0ctransactions\"j\n\x1bGetTransactionTreesResponse\x12K\n\x0ctransactions\x18\x01 \x03(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0ctransactions\"\x87\x01\n\x1eGetTransactionByEventIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12\x19\n\x08\x65vent_id\x18\x02 \x01(\tR\x07\x65ventId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"\x8e\x01\n\x19GetTransactionByIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12%\n\x0etransaction_id\x18\x02 \x01(\tR\rtransactionId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"c\n\x16GetTransactionResponse\x12I\n\x0btransaction\x18\x01 \x01(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0btransaction\"c\n\x1aGetFlatTransactionResponse\x12\x45\n\x0btransaction\x18\x01 \x01(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0btransaction\"2\n\x13GetLedgerEndRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\"T\n\x14GetLedgerEndResponse\x12<\n\x06offset\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x06offset2\xfd\x06\n\x12TransactionService\x12t\n\x0fGetTransactions\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a/.com.daml.ledger.api.v1.GetTransactionsResponse0\x01\x12|\n\x13GetTransactionTrees\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a\x33.com.daml.ledger.api.v1.GetTransactionTreesResponse0\x01\x12\x81\x01\n\x17GetTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12w\n\x12GetTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12\x89\x01\n\x1bGetFlatTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12\x7f\n\x16GetFlatTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12i\n\x0cGetLedgerEnd\x12+.com.daml.ledger.api.v1.GetLedgerEndRequest\x1a,.com.daml.ledger.api.v1.GetLedgerEndResponseB\x96\x01\n\x16\x63om.daml.ledger.api.v1B\x1cTransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_GETTRANSACTIONSREQUEST = DESCRIPTOR.message_types_by_name['GetTransactionsRequest']
_GETTRANSACTIONSRESPONSE = DESCRIPTOR.message_types_by_name['GetTransactionsResponse']
_GETTRANSACTIONTREESRESPONSE = DESCRIPTOR.message_types_by_name['GetTransactionTreesResponse']
_GETTRANSACTIONBYEVENTIDREQUEST = DESCRIPTOR.message_types_by_name['GetTransactionByEventIdRequest']
_GETTRANSACTIONBYIDREQUEST = DESCRIPTOR.message_types_by_name['GetTransactionByIdRequest']
_GETTRANSACTIONRESPONSE = DESCRIPTOR.message_types_by_name['GetTransactionResponse']
_GETFLATTRANSACTIONRESPONSE = DESCRIPTOR.message_types_by_name['GetFlatTransactionResponse']
_GETLEDGERENDREQUEST = DESCRIPTOR.message_types_by_name['GetLedgerEndRequest']
_GETLEDGERENDRESPONSE = DESCRIPTOR.message_types_by_name['GetLedgerEndResponse']
GetTransactionsRequest = _reflection.GeneratedProtocolMessageType('GetTransactionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionsRequest)
  })
_sym_db.RegisterMessage(GetTransactionsRequest)

GetTransactionsResponse = _reflection.GeneratedProtocolMessageType('GetTransactionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionsResponse)
  })
_sym_db.RegisterMessage(GetTransactionsResponse)

GetTransactionTreesResponse = _reflection.GeneratedProtocolMessageType('GetTransactionTreesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONTREESRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionTreesResponse)
  })
_sym_db.RegisterMessage(GetTransactionTreesResponse)

GetTransactionByEventIdRequest = _reflection.GeneratedProtocolMessageType('GetTransactionByEventIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONBYEVENTIDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionByEventIdRequest)
  })
_sym_db.RegisterMessage(GetTransactionByEventIdRequest)

GetTransactionByIdRequest = _reflection.GeneratedProtocolMessageType('GetTransactionByIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONBYIDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionByIdRequest)
  })
_sym_db.RegisterMessage(GetTransactionByIdRequest)

GetTransactionResponse = _reflection.GeneratedProtocolMessageType('GetTransactionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionResponse)
  })
_sym_db.RegisterMessage(GetTransactionResponse)

GetFlatTransactionResponse = _reflection.GeneratedProtocolMessageType('GetFlatTransactionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFLATTRANSACTIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetFlatTransactionResponse)
  })
_sym_db.RegisterMessage(GetFlatTransactionResponse)

GetLedgerEndRequest = _reflection.GeneratedProtocolMessageType('GetLedgerEndRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERENDREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerEndRequest)
  })
_sym_db.RegisterMessage(GetLedgerEndRequest)

GetLedgerEndResponse = _reflection.GeneratedProtocolMessageType('GetLedgerEndResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERENDRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.transaction_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerEndResponse)
  })
_sym_db.RegisterMessage(GetLedgerEndResponse)

_TRANSACTIONSERVICE = DESCRIPTOR.services_by_name['TransactionService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\034TransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _GETTRANSACTIONSREQUEST._serialized_start=212
  _GETTRANSACTIONSREQUEST._serialized_end=474
  _GETTRANSACTIONSRESPONSE._serialized_start=476
  _GETTRANSACTIONSRESPONSE._serialized_end=574
  _GETTRANSACTIONTREESRESPONSE._serialized_start=576
  _GETTRANSACTIONTREESRESPONSE._serialized_end=682
  _GETTRANSACTIONBYEVENTIDREQUEST._serialized_start=685
  _GETTRANSACTIONBYEVENTIDREQUEST._serialized_end=820
  _GETTRANSACTIONBYIDREQUEST._serialized_start=823
  _GETTRANSACTIONBYIDREQUEST._serialized_end=965
  _GETTRANSACTIONRESPONSE._serialized_start=967
  _GETTRANSACTIONRESPONSE._serialized_end=1066
  _GETFLATTRANSACTIONRESPONSE._serialized_start=1068
  _GETFLATTRANSACTIONRESPONSE._serialized_end=1167
  _GETLEDGERENDREQUEST._serialized_start=1169
  _GETLEDGERENDREQUEST._serialized_end=1219
  _GETLEDGERENDRESPONSE._serialized_start=1221
  _GETLEDGERENDRESPONSE._serialized_end=1305
  _TRANSACTIONSERVICE._serialized_start=1308
  _TRANSACTIONSERVICE._serialized_end=2201
# @@protoc_insertion_point(module_scope)
