# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/daml_lf_1_14/daml_lf.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import daml_lf_1_pb2 as com_dot_daml_dot_daml__lf__1__14_dot_daml__lf__1__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#com/daml/daml_lf_1_14/daml_lf.proto\x12\tdaml_lf_1\x1a%com/daml/daml_lf_1_14/daml_lf_1.proto\"m\n\x0e\x41rchivePayload\x12\x14\n\x05minor\x18\x03 \x01(\tR\x05minor\x12\x30\n\tdaml_lf_1\x18\x02 \x01(\x0b\x32\x12.daml_lf_1.PackageH\x00R\x07\x64\x61mlLf1B\x05\n\x03SumJ\x06\x08\x8fN\x10\x90NJ\x04\x08\x01\x10\x02\"u\n\x07\x41rchive\x12<\n\rhash_function\x18\x01 \x01(\x0e\x32\x17.daml_lf_1.HashFunctionR\x0chashFunction\x12\x18\n\x07payload\x18\x03 \x01(\x0cR\x07payload\x12\x12\n\x04hash\x18\x04 \x01(\tR\x04hash*\x1a\n\x0cHashFunction\x12\n\n\x06SHA256\x10\x00\x42|\n\x15\x63om.daml.daml_lf_1_14ZDgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/daml_lf_1_14\xaa\x02\x1c\x43om.Daml.Daml_Lf_1_14.DamlLfb\x06proto3')

_HASHFUNCTION = DESCRIPTOR.enum_types_by_name['HashFunction']
HashFunction = enum_type_wrapper.EnumTypeWrapper(_HASHFUNCTION)
SHA256 = 0


_ARCHIVEPAYLOAD = DESCRIPTOR.message_types_by_name['ArchivePayload']
_ARCHIVE = DESCRIPTOR.message_types_by_name['Archive']
ArchivePayload = _reflection.GeneratedProtocolMessageType('ArchivePayload', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEPAYLOAD,
  '__module__' : 'com.daml.daml_lf_1_14.daml_lf_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.ArchivePayload)
  })
_sym_db.RegisterMessage(ArchivePayload)

Archive = _reflection.GeneratedProtocolMessageType('Archive', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVE,
  '__module__' : 'com.daml.daml_lf_1_14.daml_lf_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Archive)
  })
_sym_db.RegisterMessage(Archive)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.daml.daml_lf_1_14ZDgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/daml_lf_1_14\252\002\034Com.Daml.Daml_Lf_1_14.DamlLf'
  _HASHFUNCTION._serialized_start=319
  _HASHFUNCTION._serialized_end=345
  _ARCHIVEPAYLOAD._serialized_start=89
  _ARCHIVEPAYLOAD._serialized_end=198
  _ARCHIVE._serialized_start=200
  _ARCHIVE._serialized_end=317
# @@protoc_insertion_point(module_scope)
