# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/user_management_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:com/daml/ledger/api/v1/admin/user_management_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\";\n\x04User\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12#\n\rprimary_party\x18\x02 \x01(\tR\x0cprimaryParty\"\xec\x02\n\x05Right\x12\x63\n\x11participant_admin\x18\x01 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.admin.Right.ParticipantAdminH\x00R\x10participantAdmin\x12L\n\ncan_act_as\x18\x02 \x01(\x0b\x32,.com.daml.ledger.api.v1.admin.Right.CanActAsH\x00R\x08\x63\x61nActAs\x12O\n\x0b\x63\x61n_read_as\x18\x03 \x01(\x0b\x32-.com.daml.ledger.api.v1.admin.Right.CanReadAsH\x00R\tcanReadAs\x1a\x12\n\x10ParticipantAdmin\x1a \n\x08\x43\x61nActAs\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x1a!\n\tCanReadAs\x12\x14\n\x05party\x18\x01 \x01(\tR\x05partyB\x06\n\x04kind\"\x88\x01\n\x11\x43reateUserRequest\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x04user\x12;\n\x06rights\x18\x02 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights\"L\n\x12\x43reateUserResponse\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x04user\")\n\x0eGetUserRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"I\n\x0fGetUserResponse\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x04user\",\n\x11\x44\x65leteUserRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"\x14\n\x12\x44\x65leteUserResponse\"N\n\x10ListUsersRequest\x12\x1d\n\npage_token\x18\x02 \x01(\tR\tpageToken\x12\x1b\n\tpage_size\x18\x03 \x01(\x05R\x08pageSize\"u\n\x11ListUsersResponse\x12\x38\n\x05users\x18\x01 \x03(\x0b\x32\".com.daml.ledger.api.v1.admin.UserR\x05users\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"n\n\x16GrantUserRightsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12;\n\x06rights\x18\x02 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights\"p\n\x17GrantUserRightsResponse\x12U\n\x14newly_granted_rights\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x12newlyGrantedRights\"o\n\x17RevokeUserRightsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12;\n\x06rights\x18\x02 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights\"q\n\x18RevokeUserRightsResponse\x12U\n\x14newly_revoked_rights\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x12newlyRevokedRights\"0\n\x15ListUserRightsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"U\n\x16ListUserRightsResponse\x12;\n\x06rights\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.admin.RightR\x06rights2\xd0\x06\n\x15UserManagementService\x12o\n\nCreateUser\x12/.com.daml.ledger.api.v1.admin.CreateUserRequest\x1a\x30.com.daml.ledger.api.v1.admin.CreateUserResponse\x12\x66\n\x07GetUser\x12,.com.daml.ledger.api.v1.admin.GetUserRequest\x1a-.com.daml.ledger.api.v1.admin.GetUserResponse\x12o\n\nDeleteUser\x12/.com.daml.ledger.api.v1.admin.DeleteUserRequest\x1a\x30.com.daml.ledger.api.v1.admin.DeleteUserResponse\x12l\n\tListUsers\x12..com.daml.ledger.api.v1.admin.ListUsersRequest\x1a/.com.daml.ledger.api.v1.admin.ListUsersResponse\x12~\n\x0fGrantUserRights\x12\x34.com.daml.ledger.api.v1.admin.GrantUserRightsRequest\x1a\x35.com.daml.ledger.api.v1.admin.GrantUserRightsResponse\x12\x81\x01\n\x10RevokeUserRights\x12\x35.com.daml.ledger.api.v1.admin.RevokeUserRightsRequest\x1a\x36.com.daml.ledger.api.v1.admin.RevokeUserRightsResponse\x12{\n\x0eListUserRights\x12\x33.com.daml.ledger.api.v1.admin.ListUserRightsRequest\x1a\x34.com.daml.ledger.api.v1.admin.ListUserRightsResponseB\xab\x01\n\x1c\x63om.daml.ledger.api.v1.adminB\x1fUserManagementServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3')



_USER = DESCRIPTOR.message_types_by_name['User']
_RIGHT = DESCRIPTOR.message_types_by_name['Right']
_RIGHT_PARTICIPANTADMIN = _RIGHT.nested_types_by_name['ParticipantAdmin']
_RIGHT_CANACTAS = _RIGHT.nested_types_by_name['CanActAs']
_RIGHT_CANREADAS = _RIGHT.nested_types_by_name['CanReadAs']
_CREATEUSERREQUEST = DESCRIPTOR.message_types_by_name['CreateUserRequest']
_CREATEUSERRESPONSE = DESCRIPTOR.message_types_by_name['CreateUserResponse']
_GETUSERREQUEST = DESCRIPTOR.message_types_by_name['GetUserRequest']
_GETUSERRESPONSE = DESCRIPTOR.message_types_by_name['GetUserResponse']
_DELETEUSERREQUEST = DESCRIPTOR.message_types_by_name['DeleteUserRequest']
_DELETEUSERRESPONSE = DESCRIPTOR.message_types_by_name['DeleteUserResponse']
_LISTUSERSREQUEST = DESCRIPTOR.message_types_by_name['ListUsersRequest']
_LISTUSERSRESPONSE = DESCRIPTOR.message_types_by_name['ListUsersResponse']
_GRANTUSERRIGHTSREQUEST = DESCRIPTOR.message_types_by_name['GrantUserRightsRequest']
_GRANTUSERRIGHTSRESPONSE = DESCRIPTOR.message_types_by_name['GrantUserRightsResponse']
_REVOKEUSERRIGHTSREQUEST = DESCRIPTOR.message_types_by_name['RevokeUserRightsRequest']
_REVOKEUSERRIGHTSRESPONSE = DESCRIPTOR.message_types_by_name['RevokeUserRightsResponse']
_LISTUSERRIGHTSREQUEST = DESCRIPTOR.message_types_by_name['ListUserRightsRequest']
_LISTUSERRIGHTSRESPONSE = DESCRIPTOR.message_types_by_name['ListUserRightsResponse']
User = _reflection.GeneratedProtocolMessageType('User', (_message.Message,), {
  'DESCRIPTOR' : _USER,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.User)
  })
_sym_db.RegisterMessage(User)

Right = _reflection.GeneratedProtocolMessageType('Right', (_message.Message,), {

  'ParticipantAdmin' : _reflection.GeneratedProtocolMessageType('ParticipantAdmin', (_message.Message,), {
    'DESCRIPTOR' : _RIGHT_PARTICIPANTADMIN,
    '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
    # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right.ParticipantAdmin)
    })
  ,

  'CanActAs' : _reflection.GeneratedProtocolMessageType('CanActAs', (_message.Message,), {
    'DESCRIPTOR' : _RIGHT_CANACTAS,
    '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
    # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right.CanActAs)
    })
  ,

  'CanReadAs' : _reflection.GeneratedProtocolMessageType('CanReadAs', (_message.Message,), {
    'DESCRIPTOR' : _RIGHT_CANREADAS,
    '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
    # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right.CanReadAs)
    })
  ,
  'DESCRIPTOR' : _RIGHT,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.Right)
  })
_sym_db.RegisterMessage(Right)
_sym_db.RegisterMessage(Right.ParticipantAdmin)
_sym_db.RegisterMessage(Right.CanActAs)
_sym_db.RegisterMessage(Right.CanReadAs)

CreateUserRequest = _reflection.GeneratedProtocolMessageType('CreateUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUSERREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.CreateUserRequest)
  })
_sym_db.RegisterMessage(CreateUserRequest)

CreateUserResponse = _reflection.GeneratedProtocolMessageType('CreateUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUSERRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.CreateUserResponse)
  })
_sym_db.RegisterMessage(CreateUserResponse)

GetUserRequest = _reflection.GeneratedProtocolMessageType('GetUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETUSERREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetUserRequest)
  })
_sym_db.RegisterMessage(GetUserRequest)

GetUserResponse = _reflection.GeneratedProtocolMessageType('GetUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETUSERRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetUserResponse)
  })
_sym_db.RegisterMessage(GetUserResponse)

DeleteUserRequest = _reflection.GeneratedProtocolMessageType('DeleteUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEUSERREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.DeleteUserRequest)
  })
_sym_db.RegisterMessage(DeleteUserRequest)

DeleteUserResponse = _reflection.GeneratedProtocolMessageType('DeleteUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEUSERRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.DeleteUserResponse)
  })
_sym_db.RegisterMessage(DeleteUserResponse)

ListUsersRequest = _reflection.GeneratedProtocolMessageType('ListUsersRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUsersRequest)
  })
_sym_db.RegisterMessage(ListUsersRequest)

ListUsersResponse = _reflection.GeneratedProtocolMessageType('ListUsersResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUsersResponse)
  })
_sym_db.RegisterMessage(ListUsersResponse)

GrantUserRightsRequest = _reflection.GeneratedProtocolMessageType('GrantUserRightsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GRANTUSERRIGHTSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GrantUserRightsRequest)
  })
_sym_db.RegisterMessage(GrantUserRightsRequest)

GrantUserRightsResponse = _reflection.GeneratedProtocolMessageType('GrantUserRightsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GRANTUSERRIGHTSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GrantUserRightsResponse)
  })
_sym_db.RegisterMessage(GrantUserRightsResponse)

RevokeUserRightsRequest = _reflection.GeneratedProtocolMessageType('RevokeUserRightsRequest', (_message.Message,), {
  'DESCRIPTOR' : _REVOKEUSERRIGHTSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.RevokeUserRightsRequest)
  })
_sym_db.RegisterMessage(RevokeUserRightsRequest)

RevokeUserRightsResponse = _reflection.GeneratedProtocolMessageType('RevokeUserRightsResponse', (_message.Message,), {
  'DESCRIPTOR' : _REVOKEUSERRIGHTSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.RevokeUserRightsResponse)
  })
_sym_db.RegisterMessage(RevokeUserRightsResponse)

ListUserRightsRequest = _reflection.GeneratedProtocolMessageType('ListUserRightsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERRIGHTSREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUserRightsRequest)
  })
_sym_db.RegisterMessage(ListUserRightsRequest)

ListUserRightsResponse = _reflection.GeneratedProtocolMessageType('ListUserRightsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERRIGHTSRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.user_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.ListUserRightsResponse)
  })
_sym_db.RegisterMessage(ListUserRightsResponse)

_USERMANAGEMENTSERVICE = DESCRIPTOR.services_by_name['UserManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.daml.ledger.api.v1.adminB\037UserManagementServiceOuterClassZKgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1/admin\252\002\034Com.Daml.Ledger.Api.V1.Admin'
  _USER._serialized_start=92
  _USER._serialized_end=151
  _RIGHT._serialized_start=154
  _RIGHT._serialized_end=518
  _RIGHT_PARTICIPANTADMIN._serialized_start=423
  _RIGHT_PARTICIPANTADMIN._serialized_end=441
  _RIGHT_CANACTAS._serialized_start=443
  _RIGHT_CANACTAS._serialized_end=475
  _RIGHT_CANREADAS._serialized_start=477
  _RIGHT_CANREADAS._serialized_end=510
  _CREATEUSERREQUEST._serialized_start=521
  _CREATEUSERREQUEST._serialized_end=657
  _CREATEUSERRESPONSE._serialized_start=659
  _CREATEUSERRESPONSE._serialized_end=735
  _GETUSERREQUEST._serialized_start=737
  _GETUSERREQUEST._serialized_end=778
  _GETUSERRESPONSE._serialized_start=780
  _GETUSERRESPONSE._serialized_end=853
  _DELETEUSERREQUEST._serialized_start=855
  _DELETEUSERREQUEST._serialized_end=899
  _DELETEUSERRESPONSE._serialized_start=901
  _DELETEUSERRESPONSE._serialized_end=921
  _LISTUSERSREQUEST._serialized_start=923
  _LISTUSERSREQUEST._serialized_end=1001
  _LISTUSERSRESPONSE._serialized_start=1003
  _LISTUSERSRESPONSE._serialized_end=1120
  _GRANTUSERRIGHTSREQUEST._serialized_start=1122
  _GRANTUSERRIGHTSREQUEST._serialized_end=1232
  _GRANTUSERRIGHTSRESPONSE._serialized_start=1234
  _GRANTUSERRIGHTSRESPONSE._serialized_end=1346
  _REVOKEUSERRIGHTSREQUEST._serialized_start=1348
  _REVOKEUSERRIGHTSREQUEST._serialized_end=1459
  _REVOKEUSERRIGHTSRESPONSE._serialized_start=1461
  _REVOKEUSERRIGHTSRESPONSE._serialized_end=1574
  _LISTUSERRIGHTSREQUEST._serialized_start=1576
  _LISTUSERRIGHTSREQUEST._serialized_end=1624
  _LISTUSERRIGHTSRESPONSE._serialized_start=1626
  _LISTUSERRIGHTSRESPONSE._serialized_end=1711
  _USERMANAGEMENTSERVICE._serialized_start=1714
  _USERMANAGEMENTSERVICE._serialized_end=2562
# @@protoc_insertion_point(module_scope)
