# Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/event.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import value_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"com/daml/ledger/api/v1/event.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\"com/daml/ledger/api/v1/value.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\xa8\x01\n\x05\x45vent\x12@\n\x07\x63reated\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.CreatedEventH\x00R\x07\x63reated\x12\x43\n\x08\x61rchived\x18\x03 \x01(\x0b\x32%.com.daml.ledger.api.v1.ArchivedEventH\x00R\x08\x61rchivedB\x07\n\x05\x65ventJ\x04\x08\x02\x10\x03R\texercised\"\xca\x03\n\x0c\x43reatedEvent\x12\x19\n\x08\x65vent_id\x18\x01 \x01(\tR\x07\x65ventId\x12\x1f\n\x0b\x63ontract_id\x18\x02 \x01(\tR\ncontractId\x12\x43\n\x0btemplate_id\x18\x03 \x01(\x0b\x32\".com.daml.ledger.api.v1.IdentifierR\ntemplateId\x12@\n\x0c\x63ontract_key\x18\x07 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueR\x0b\x63ontractKey\x12I\n\x10\x63reate_arguments\x18\x04 \x01(\x0b\x32\x1e.com.daml.ledger.api.v1.RecordR\x0f\x63reateArguments\x12\'\n\x0fwitness_parties\x18\x05 \x03(\tR\x0ewitnessParties\x12 \n\x0bsignatories\x18\x08 \x03(\tR\x0bsignatories\x12\x1c\n\tobservers\x18\t \x03(\tR\tobservers\x12\x43\n\x0e\x61greement_text\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.StringValueR\ragreementText\"\xb9\x01\n\rArchivedEvent\x12\x19\n\x08\x65vent_id\x18\x01 \x01(\tR\x07\x65ventId\x12\x1f\n\x0b\x63ontract_id\x18\x02 \x01(\tR\ncontractId\x12\x43\n\x0btemplate_id\x18\x03 \x01(\x0b\x32\".com.daml.ledger.api.v1.IdentifierR\ntemplateId\x12\'\n\x0fwitness_parties\x18\x04 \x03(\tR\x0ewitnessParties\"\xdb\x03\n\x0e\x45xercisedEvent\x12\x19\n\x08\x65vent_id\x18\x01 \x01(\tR\x07\x65ventId\x12\x1f\n\x0b\x63ontract_id\x18\x02 \x01(\tR\ncontractId\x12\x43\n\x0btemplate_id\x18\x03 \x01(\x0b\x32\".com.daml.ledger.api.v1.IdentifierR\ntemplateId\x12\x16\n\x06\x63hoice\x18\x05 \x01(\tR\x06\x63hoice\x12\x46\n\x0f\x63hoice_argument\x18\x06 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueR\x0e\x63hoiceArgument\x12%\n\x0e\x61\x63ting_parties\x18\x07 \x03(\tR\ractingParties\x12\x1c\n\tconsuming\x18\x08 \x01(\x08R\tconsuming\x12\'\n\x0fwitness_parties\x18\n \x03(\tR\x0ewitnessParties\x12&\n\x0f\x63hild_event_ids\x18\x0b \x03(\tR\rchildEventIds\x12\x46\n\x0f\x65xercise_result\x18\x0c \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueR\x0e\x65xerciseResultJ\x04\x08\x04\x10\x05J\x04\x08\t\x10\nB\x89\x01\n\x16\x63om.daml.ledger.api.v1B\x0f\x45ventOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')



_EVENT = DESCRIPTOR.message_types_by_name['Event']
_CREATEDEVENT = DESCRIPTOR.message_types_by_name['CreatedEvent']
_ARCHIVEDEVENT = DESCRIPTOR.message_types_by_name['ArchivedEvent']
_EXERCISEDEVENT = DESCRIPTOR.message_types_by_name['ExercisedEvent']
Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Event)
  })
_sym_db.RegisterMessage(Event)

CreatedEvent = _reflection.GeneratedProtocolMessageType('CreatedEvent', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDEVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.CreatedEvent)
  })
_sym_db.RegisterMessage(CreatedEvent)

ArchivedEvent = _reflection.GeneratedProtocolMessageType('ArchivedEvent', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEDEVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ArchivedEvent)
  })
_sym_db.RegisterMessage(ArchivedEvent)

ExercisedEvent = _reflection.GeneratedProtocolMessageType('ExercisedEvent', (_message.Message,), {
  'DESCRIPTOR' : _EXERCISEDEVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ExercisedEvent)
  })
_sym_db.RegisterMessage(ExercisedEvent)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\017EventOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _EVENT._serialized_start=131
  _EVENT._serialized_end=299
  _CREATEDEVENT._serialized_start=302
  _CREATEDEVENT._serialized_end=760
  _ARCHIVEDEVENT._serialized_start=763
  _ARCHIVEDEVENT._serialized_end=948
  _EXERCISEDEVENT._serialized_start=951
  _EXERCISEDEVENT._serialized_end=1426
# @@protoc_insertion_point(module_scope)
